import os


def _parse_value(value):
    if isinstance(value, list):
        return [_parse_value(i) for i in value]

    if value.isdigit():
        return int(value)
    try:
        return float(value)
    except ValueError:
        pass
    return value


def _parse_nested_env(key, value, level_sep="_", list_sep=","):
    split = key.split(level_sep)
    if len(split) == 1:
        if list_sep in value:
            value = value.split(list_sep)
        return {key: _parse_value(value)}
    return {
        split[0]: _parse_nested_env(level_sep.join(split[1:]), value),
    }


def parse_env(prefix=None, env=os.environ):
    env_config = None
    if prefix is not None:
        prefix = prefix.lower()
        env_config = {k.lower().split(prefix)[-1]: v for k, v in env.items() if k.lower().startswith(prefix)}
    else:
        env_config = env

    env_config_parsed = {}
    for k, v in env_config.items():
        nested = _parse_nested_env(k, v)
        env_config_parsed.update(nested)

    return env_config_parsed



import os
from copy import deepcopy

import yaml

from logtron.util import parse_env

ENV_PREFIX = "LOGTRON_"
DEFAULT_CONFIG_FILE = "logtron.yaml"
DEFAULT_CONFIG = {
    "handlers": [
        "logtron.handlers.ConsoleHandler",
    ],
}


def read_config_file(path):
    if os.path.isfile(path):
        with open(path, "r") as f:
            return yaml.load(f, Loader=yaml.SafeLoader)
    return {}


def discover_config(existing=None):
    config = deepcopy(DEFAULT_CONFIG)
    if existing is None:
        existing = {}

    # Read default config file
    config.update(read_config_file(DEFAULT_CONFIG_FILE))

    # Read env vars
    config.update(parse_env(ENV_PREFIX))

    # Read explicit config
    if isinstance(existing, str):
        config.update(read_config_file(existing))
    else:
        config.update(existing)

    return config



import importlib
import logging

from logtron.config import discover_config as discover_config_base
from logtron.formatters import JsonFormatter
from logtron.util import flatten_dict

is_configured = False


def __get_handlers(config, formatter):
    config = {k.lower(): v for k, v in config.items()}

    handlers = [(i,) + tuple(i.rsplit(".", 1)) for i in config["handlers"]]
    classes = [i[2] for i in handlers]

    for handler, module_name, class_name in handlers:
        HandlerClass = getattr(importlib.import_module(module_name), class_name)
        instance = None
        args = {}
        if config.get(handler.lower()) is not None:
            args.update(config[handler.lower()])
        if classes.count(class_name) == 1 and config.get(class_name.lower()) is not None:
            args.update(config[class_name.lower()])
        instance = HandlerClass(**args)
        instance.setFormatter(formatter)
        yield instance


def autodiscover(name=None, level=logging.INFO, **kwargs):
    global is_configured

    refresh = kwargs.get("refresh", False)
    if not refresh and is_configured:
        return logging.getLogger(name)

    root_logger = logging.getLogger()
    root_logger.setLevel(level)
    existing_handlers = root_logger.handlers
    [root_logger.removeHandler(i) for i in existing_handlers]

    discover_config = kwargs.get("discover_config", discover_config_base)
    config = discover_config(kwargs.get("config"))

    context = config.get("context", {})
    formatter = JsonFormatter(
        discover_context=kwargs.get("discover_context", lambda: context),
        flatten=kwargs.get("flatten", False),
    )
    handlers = __get_handlers(config, formatter)
    for i in handlers:
        root_logger.addHandler(i)

    is_configured = True

    return logging.getLogger(name)



import json
import traceback
from logging import Formatter, LogRecord

from logtron.util import flatten_dict


class JsonFormatter(Formatter):
    def __init__(self, **kwargs):
        super(JsonFormatter, self).__init__()
        self.reserved = ["message", "asctime"] + [
            k for k, v in LogRecord(None, None, None, None, None, None, None).__dict__.items()
        ]
        self.flatten = kwargs.get("flatten", False)
        self.discover_context = kwargs.get("discover_context", lambda: {})

    def format(self, record):
        data = {
            "timestamp": int(record.created * 1000),
            "message": record.msg,
            "name": record.name,
            "level": record.levelno,
        }

        if record.exc_info is not None:
            data["exception"] = "".join(traceback.format_exception(*record.exc_info))

        if self.flatten:
            items = {
                k: v
                for d in [record.__dict__, flatten_dict(self.discover_context(), "context")]
                for k, v in d.items()
                if k not in self.reserved and k not in data
            }
            data.update(flatten_dict(items))
        else:
            items = {k: v for k, v in record.__dict__.items() if k not in self.reserved and k not in data}
            data.update({"extra": items, "context": self.discover_context()})

        return json.dumps(data)



import logging

from importlib_metadata import version

from logtron.autodiscover import autodiscover

try:
    __version__ = version(__package__)
except:
    __version__ = "unspecified"


def flush(name=None):
    for i in logging.getLogger(name).handlers:
        i.flush()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logtron',
 'logtron.config',
 'logtron.formatters',
 'logtron.handlers',
 'logtron.util']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=1.7.0,<2.0.0', 'pyyaml>=5.3.1,<6.0.0']

setup_kwargs = {
    'name': 'logtron',
    'version': '0.1.13',
    'description': 'A simple logging library with JSON log formatting',
    'long_description': '# Logtron\n\n[![Release](https://img.shields.io/pypi/v/logtron?logo=python&style=flat)](https://pypi.org/project/logtron)\n[![Downloads](https://img.shields.io/pypi/dm/logtron?logo=python&style=flat)](https://pypi.org/project/logtron)\n[![Supported Versions](https://img.shields.io/pypi/pyversions/logtron.svg?logo=python&style=flat)](https://pypi.org/project/logtron)\n[![License](https://img.shields.io/github/license/ilija1/logtron?logo=apache&style=flat)](LICENSE)\n\n[![Build](https://img.shields.io/travis/ilija1/logtron?logo=travis&style=flat)](https://travis-ci.org/ilija1/logtron)\n[![Coverage](https://img.shields.io/codecov/c/gh/ilija1/logtron?logo=codecov&style=flat)](https://codecov.io/gh/ilija1/logtron)\n[![Documentation](https://img.shields.io/readthedocs/logtron?logo=read-the-docs&style=flat)](https://logtron.readthedocs.io/en/latest)\n[![Maintainability](https://img.shields.io/codeclimate/maintainability/ilija1/logtron?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron/maintainability)\n[![Tech Debt](https://img.shields.io/codeclimate/tech-debt/ilija1/logtron?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron/issues)\n[![Issues](https://img.shields.io/codeclimate/issues/ilija1/logtron?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron/issues)\n\n**Logtron** is a simple logging library with JSON log formatting.\n\n```python\n>>> import logtron\n>>> logger = logtron.autodiscover()\n>>> logger.info("hello world")\n{"timestamp": 1598900664859, "message": "hello world", "name": "root", "level": 20, "context": {}, "extra": {}}\n>>> logger.info("extra args", extra={"foo": "bar", "count": 7})\n{"timestamp": 1598900667704, "message": "extra args", "name": "root", "level": 20, "context": {}, "extra": {"foo": "bar", "count": 7}}\n>>>\n```\n\nOr\n\n```python\n>>> import logtron\n>>> logtron.autodiscover() # Only needs to run once somewhere to configure the root logger\n<RootLogger root (INFO)>\n>>>\n>>> import logging\n>>> logger = logging.getLogger()\n>>> logger.info("hello world")\n{"timestamp": 1598900735699, "message": "hello world", "name": "root", "level": 20, "context": {}, "extra": {}}\n>>> logger.info("extra args", extra={"foo": "bar", "count": 7})\n{"timestamp": 1598900757238, "message": "extra args", "name": "root", "level": 20, "context": {}, "extra": {"foo": "bar", "count": 7}}\n>>>\n```\n\nLogtron allows you to skip all the usual boilerplate when configuring python logging.\n\nLogtron will default to a console JSON log formatter that is compatible with popular log aggregators such as [Logstash](https://www.elastic.co/guide/en/logstash/current/introduction.html), [Fluent Bit](https://docs.fluentbit.io/manual/), or [AWS CloudWatch Logs](https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/WhatIsCloudWatchLogs.html).\n\n## Installing Logtron and Supported Versions\n\nLogtron is available on PyPI:\n\n```shell\n$ python -m pip install logtron\n```\n\nLogtron officially supports Python 2.7 & 3.5+.\n',
    'author': 'Ilija Stevcev',
    'author_email': 'ilija1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ilija1/logtron/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)
